Attribute VB_Name = "Errors"
Option Explicit

'Error categories
Public Const ErrCatMath = 1
Public Const ErrCatFile = 2
Public Const ErrCatMisc = 3
Public Const ErrCatIllegal = 0

'Math errors
Public Const ErrBadCall = 5
Public Const ErrOverFlow = 6
Public Const ErrDivByZero = 11
Public Const ErrTypeMismatch = 13

'File and device errors
Public Const ErrBadFileNameOrNumber = 52
Public Const ErrFileNotFound = 53
Public Const ErrBadFileMode = 54
Public Const ErrFileAlreadyOpen = 55
Public Const ErrDeviceIO = 57
Public Const ErrFileAlreadyExist = 58
Public Const ErrDiskFull = 61
Public Const ErrInputPastEndOfFile = 63
Public Const ErrBadFileName = 64
Public Const ErrDeviceUnavail = 68
Public Const ErrDiskNotReady = 71
Public Const ErrPathFileAccessError = 75
Public Const ErrPathNotExist = 76

'Misc
Public Const ErrCancel = 32755

Private Function addstr(ByVal str As String)
    If str = "" Then
        addstr = "."
    Else
        addstr = "on file " & str & "."
    End If
End Function

Public Function ErrCategory(ByVal errno As Single) As Integer
' returns ErrCatMath, ErrCatFile, ErrCatMisc, ErrCatIllegal
    If errno >= 5 And errno <= 13 Then
        ErrCategory = ErrCatMath
    ElseIf errno >= 52 And errno <= 76 Then
        ErrCategory = ErrCatFile
    ElseIf errno = 32755 Then
        ErrCategory = ErrCatMisc
    Else
        ErrCategory = ErrCatIllegal
    End If
End Function

Public Function FileError(ByVal errno As Single, Optional ByVal capt As String = "", Optional ByVal str As String = "") As Integer
' returns vbRetry or vbCancel

    Dim txt As String
    Dim caption As String
    
    txt = ""
    If capt = "" Then
        caption = "Error"
    Else
        caption = capt
    End If
    
    Select Case errno
    Case ErrBadFileNameOrNumber
        txt = "Bad filename or number occurred" & addstr(str)
    Case ErrFileNotFound
        If str = "" Then
            txt = "File not found."
        Else
            txt = "File " & str & " not found."
        End If
    Case ErrBadFileMode
        txt = "Bad file mode occurred" & addstr(str)
    Case ErrFileAlreadyOpen
        txt = "File already open occurred" & addstr(str)
        txt = txt & " This is almost always an internal error. If you"
        txt = txt & " are sure that this file is not opened by another application"
        txt = txt & " please contact the author."
    Case ErrDeviceIO
        txt = "An device I/O error occurred" & addstr(str)
        txt = txt & "This is most often caused by a malfunctioning network device."
    Case ErrDiskFull
        txt = "Disk full! Please remove some files."
    Case ErrInputPastEndOfFile
        txt = "Input past end of file occurred" & addstr(str)
        txt = txt & " This is almost always an internal error."
        txt = txt & " Please contact the author."
    Case ErrBadFileName
        txt = "Bad file name" & addstr(str)
        txt = txt & " This may be caused by illegal characters in a filename"
    Case ErrDeviceUnavail
        txt = "Device unavailable" & addstr(str)
        txt = txt & "This is most often caused by a malfunctioning network device, hard disk or floppy."
    Case ErrPathFileAccessError
        txt = "Path file access error" & addstr(str)
        txt = txt & " This error may be caused by trying to write to a write-protected file."
    Case ErrPathNotExist
        If str = "" Then
            txt = "Path does not exist."
        Else
            txt = "Path " & str & " does not exist."
        End If
    Case ErrDiskNotReady
        txt = "Disk not ready!"
    Case Else
        txt = "Undefined file error #" & Err.Number & ". Please contact the author with information about "
        txt = txt & "this error and what caused it."
    End Select
    
    FileError = MsgBox(txt, vbRetryCancel + vbExclamation, caption)

End Function

Public Function MiscError(ByVal errno As Single, Optional ByVal capt As String = "") As Integer
' returns vbRetry or vbCancel

    Dim txt As String
    Dim caption As String
    
    txt = ""
    If capt = "" Then
        caption = "Error"
    Else
        caption = capt
    End If
    
    ' strange error!
    If errno < -2140000000 Then
        txt = "Drive not ready!"
    Else
        txt = "Undefined file error #" & errno & ". Please contact the author with information about "
        txt = txt & "this error and what caused it."
    End If
    
    MiscError = MsgBox(txt, vbRetryCancel + vbExclamation, capt)
End Function
